@@ -11,7 +11,6 @@
 #include "state.h"
 struct astime_state state;
 
-#include "background.xpm"
 #include "dof.xpm"
 #include "months.xpm"
 #include "digits.xpm"
@@ -23,7 +22,6 @@ typedef struct _XpmIcon {
     XpmAttributes attributes;
 } XpmIcon;
 
-XpmIcon backgroundXpm;
 XpmIcon dofXpm;
 XpmIcon monthsXpm;
 XpmIcon digitsXpm;
@@ -31,6 +29,8 @@ XpmIcon digitsXpm;
 /* X windows related global variables */
 Display * mainDisplay = 0;      /* The display we are working on */
 Window Root;                    /* The root window of X11 */
+Pixmap backgroundWindow;        /* Window for drawing */
+Pixmap drawWindow;              /* Window for drawing */
 Window mainWindow;              /* Application window */
 Window iconWindow;              /* Icon window */
 XGCValues mainGCV;              /* graphics context values */
@@ -44,6 +44,14 @@ char dof_pixmap_color[2][50];
 char digits_pixmap_color[2][50];
 char months_pixmap_color[2][50];
 
+Pixel back_pix, fore_pix;
+
+/* The center of the clock */
+struct coordinates center;
+/* The half-size of the watch drawable area 
+ * (2 more pixels are given to the frame */
+struct coordinates watch_size;
+
 /* calculated values in pixels */
 int pixelLength[3];
 struct line_struct {
@@ -72,17 +80,17 @@ void draw_window(Window win)
 
         XCopyArea(
                 mainDisplay,
-                backgroundXpm.pixmap,
+                backgroundWindow,
                 win,
                 mainGC,
                 0,
                 0,
-                backgroundXpm.attributes.width,
-                backgroundXpm.attributes.height,
+                state.total_size.x,
+                state.total_size.y,
                 0,
                 0
                 );
-	if ( state.draw_day ) {
+	if ( state.draw_date ) {
 		XCopyArea(
 			mainDisplay,
 			dofXpm.pixmap,
@@ -107,7 +115,7 @@ void draw_window(Window win)
 			7,
 			8,
 			1,
-			45
+			state.total_size.y - 9
 			);
 		XCopyArea(
 			mainDisplay,
@@ -119,7 +127,7 @@ void draw_window(Window win)
 			7,
 			8,
 			8,
-			45
+			state.total_size.y - 9
 			);
 		XCopyArea(
 			mainDisplay,
@@ -131,8 +139,10 @@ void draw_window(Window win)
 			19,
 			7,
 			18,
-			46
+			state.total_size.y - 8
 			);
+		/* The year won't fit in smaller displays */
+		if (state.total_size.x > 51) {
 		XCopyArea(
 			mainDisplay,
 			digitsXpm.pixmap,
@@ -143,7 +153,7 @@ void draw_window(Window win)
 			7,
 			8,
 			38,
-			45
+			state.total_size.y - 9
 			);
 		XCopyArea(
 			mainDisplay,
@@ -155,8 +165,9 @@ void draw_window(Window win)
 			7,
 			8,
 			45,
-			45
+			state.total_size.y - 9
 			);
+		}
 	}
 #ifdef DEBUG
 	mainGCV.foreground = pix[2];
@@ -170,37 +181,37 @@ void draw_window(Window win)
 		mainDisplay,
 		win,
 		mainGC,
-		10,
-		3,
-		50,
-		3
+		center.x - watch_size.x,
+		center.y - watch_size.y,
+		center.x + watch_size.x,
+		center.y - watch_size.y
 		);
 	XDrawLine(
 		mainDisplay,
 		win,
 		mainGC,
-		50,
-		3,
-		50,
-		43
+		center.x + watch_size.x,
+		center.y - watch_size.y,
+		center.x + watch_size.x,
+		center.y + watch_size.y
 		);
 	XDrawLine(
 		mainDisplay,
 		win,
 		mainGC,
-		50,
-		43,
-		10,
-		43
+		center.x + watch_size.x,
+		center.y + watch_size.y,
+		center.x - watch_size.x,
+		center.y + watch_size.y
 		);
 	XDrawLine(
 		mainDisplay,
 		win,
 		mainGC,
-		10,
-		43,
-		10,
-		3
+		center.x - watch_size.x,
+		center.y + watch_size.y,
+		center.x - watch_size.x,
+		center.y - watch_size.y
 		);
 #endif
 	/* the pointers */
@@ -237,36 +248,36 @@ inline float convert2y(int tim)
 }
 
 /* calculate the lines to be drawn */
-void draw()
+void calculate_pointers()
 {
 	float xrate, yrate;
 	loc_time = localtime(&curTime);
 	/* seconds */
 	xrate = convert2x(loc_time->tm_sec);
 	yrate = convert2y(loc_time->tm_sec);
-	lines[0].x1 = 30 - rint(5 * xrate);
-	lines[0].y1 = 23 - rint(5 * yrate);
-	lines[0].x2 = rint(20 * xrate) + 30;
-	lines[0].y2 = rint(20 * yrate) + 23;
+	lines[0].x1 = center.x - rint(5 * xrate);
+	lines[0].y1 = center.y - rint(5 * yrate);
+	lines[0].x2 = rint(watch_size.x * xrate) + center.x;
+	lines[0].y2 = rint(watch_size.y * yrate) + center.y;
 #ifdef DEBUG
-	printf("%ld sec = %d %d -> %d %d\n", 
+	printf("%d sec = %d %d -> %d %d\n", 
 		loc_time->tm_sec, lines[0].x1, lines[0].y1,
 		lines[0].x2, lines[0].y2);
 #endif
 	/* minutes */
 	xrate = convert2x(loc_time->tm_min);
 	yrate = convert2y(loc_time->tm_min);
-	lines[1].x1 = 30 - rint(2 * xrate);
-	lines[1].y1 = 23 - rint(2 * yrate);
-	lines[1].x2 = rint(19 * xrate) + 30;
-	lines[1].y2 = rint(19 * yrate) + 23;
+	lines[1].x1 = center.x - rint(2 * xrate);
+	lines[1].y1 = center.y - rint(2 * yrate);
+	lines[1].x2 = rint((watch_size.x - 1) * xrate) + center.x;
+	lines[1].y2 = rint((watch_size.y - 1) * yrate) + center.y;
 	/* hours */
 	xrate = convert2x((loc_time->tm_hour % 12) * 5 + loc_time->tm_min/12);
 	yrate = convert2y((loc_time->tm_hour % 12) * 5 + loc_time->tm_min/12);
-	lines[2].x1 = 30 - rint(2 * xrate);
-	lines[2].y1 = 23 - rint(2 * yrate);
-	lines[2].x2 = rint(14 * xrate) + 30;
-	lines[2].y2 = rint(14 * yrate) + 23;
+	lines[2].x1 = center.x - rint(2 * xrate);
+	lines[2].y1 = center.y - rint(2 * yrate);
+	lines[2].x2 = rint((watch_size.x - 6) * xrate) + center.x;
+	lines[2].y2 = rint((watch_size.y - 6) * yrate) + center.y;
 }
 
 /*
@@ -275,7 +286,7 @@ void draw()
  */
 void refresh()
 {
-	draw();
+	calculate_pointers();
 	++update_request;
 }
 
@@ -306,6 +317,13 @@ void CheckX11Events()
                 switch(Event.type)
                 {
                 case Expose:
+#ifdef DEBUG
+			printf("Expose event caught: (%d %d) (%d x %d)\n",
+				((XExposeEvent *)&Event)->x, 
+				((XExposeEvent *)&Event)->y, 
+				((XExposeEvent *)&Event)->width, 
+				((XExposeEvent *)&Event)->height);
+#endif
                         if(Event.xexpose.count == 0) {
 				++update_request;
 			}
@@ -326,8 +344,33 @@ void CheckX11Events()
 
 void redraw()
 {
-	draw_window(mainWindow);
-	draw_window(iconWindow);
+	draw_window(drawWindow);
+/*	draw_window(mainWindow);
+	draw_window(iconWindow);*/
+        XCopyArea(
+                mainDisplay,
+                drawWindow,
+                mainWindow,
+                mainGC,
+                0,
+                0,
+		state.total_size.x,
+                state.total_size.y,
+                0,
+                0
+                );
+        XCopyArea(
+                mainDisplay,
+                drawWindow,
+                iconWindow,
+                mainGC,
+                0,
+                0,
+                state.total_size.x,
+                state.total_size.y,
+                0,
+                0
+                );
 	update_request = 0;
 }
 
@@ -355,7 +398,6 @@ void initialize(
 	int screen;
 	Status status;
 	XWindowAttributes winAttr;
-	Pixel back_pix, fore_pix;
 	XSizeHints SizeHints;
 	XTextProperty title;
 	char * app_name = "astime";
@@ -365,6 +407,50 @@ void initialize(
 	XEvent Event;
 	int color_depth;
 	int tmp;
+	struct coordinates tmp_coord;
+	float xrate, yrate;
+	Pixel tmp_pix;
+	int result;
+
+	if (strlen(mainGeometry)) {
+		/* Check the user-specified size */
+		result = XParseGeometry(
+			mainGeometry,
+			&SizeHints.x,
+			&SizeHints.y,
+			&SizeHints.width,
+			&SizeHints.height
+			);
+		if (result & WidthValue) {
+			state.total_size.x = SizeHints.width;
+		}
+		if (result & HeightValue) {
+			state.total_size.y = SizeHints.height;
+		}
+#ifdef DEBUG
+		printf("User size: %d x %d\n", 
+			state.total_size.x, state.total_size.y);
+#endif
+	}
+	if (state.draw_date) {
+/*		center.x = 30;
+		center.y = 23;
+		watch_size.x = 20;
+		watch_size.y = 20;*/
+		center.x = state.total_size.x / 2 + 3;
+		center.y = state.total_size.y / 2 - 4;
+		watch_size.x = state.total_size.x / 2 - 7;
+		watch_size.y = state.total_size.y / 2 - 7;
+	} else {
+/*		center.x = 27;
+		center.y = 27;
+		watch_size.x = 24;
+		watch_size.y = 24;*/
+		center.x = state.total_size.x / 2;
+		center.y = state.total_size.y / 2;
+		watch_size.x = center.x - 3;
+		watch_size.y = center.y - 3;
+	}
 
 	mainDisplay = XOpenDisplay(display_name);
 	if ( ! mainDisplay ) {
@@ -374,53 +460,14 @@ void initialize(
 	}
 	screen = DefaultScreen(mainDisplay);
 	Root = RootWindow(mainDisplay, screen);
-	back_pix = GetColor("white", mainDisplay, Root);
-	fore_pix = GetColor("black", mainDisplay, Root);
+	back_pix = GetColor(state.bgcolor, mainDisplay, Root);
+	fore_pix = GetColor(state.fgcolor, mainDisplay, Root);
 	color_depth = DefaultDepth(mainDisplay, screen);
 #ifdef DEBUG    
         printf("astime : detected color depth %d bpp, using %d bpp\n",
                 color_depth, color_depth);
 #endif          
-	/* adjust the background pixmap */
-	if (pushed_in) {
-		sprintf(bgpixmap_color[0],
-			". c %s",
-			DarkenCharColor(state.bgcolor, 1.6, mainDisplay, Root));
-		sprintf(bgpixmap_color[2],
-			"q c %s",
-			LightenCharColor(state.bgcolor, 2.0, mainDisplay, Root));
-	} else {
-		sprintf(bgpixmap_color[2],
-			"q c %s",
-			DarkenCharColor(state.bgcolor, 1.2, mainDisplay, Root));
-		sprintf(bgpixmap_color[0],
-			". c %s",
-			LightenCharColor(state.bgcolor, 2.0, mainDisplay, Root));
-	}
-	sprintf(bgpixmap_color[1], "c c %s", state.bgcolor);
-	sprintf(bgpixmap_color[3], "w c %s", state.fgcolor);
-	for (tmp = 0; tmp <= 3; ++tmp)
-		background[tmp+1] = bgpixmap_color[tmp];
 
-	backgroundXpm.attributes.valuemask |=
-                (XpmReturnPixels | XpmReturnExtensions);
-        status = XpmCreatePixmapFromData(
-                mainDisplay,                    /* display */
-                Root,                           /* window */
-                background,                     /* xpm */ 
-                &backgroundXpm.pixmap,          /* resulting pixmap */
-                &backgroundXpm.mask,
-                &backgroundXpm.attributes
-                );
-        if(status != XpmSuccess) {
-                printf("astime : (%d) not enough free color cells for background.\n", status);   
-		x_cleanup();
-        }       
-#ifdef DEBUG
-	printf("bg pixmap %d x %d\n", 
-		backgroundXpm.attributes.width,
-		backgroundXpm.attributes.height);
-#endif
 	/* The day-of-week pixmap */
 	sprintf(dof_pixmap_color[0], "c c %s", state.bgcolor);
 	sprintf(dof_pixmap_color[1], "l c %s", state.daycolor);
@@ -495,13 +542,34 @@ void initialize(
                 &SizeHints.height,
                 &gravity
                 );
+#ifdef DEBUG
+	printf("SizeHints were: %d x %d, setting to %d x %d\n",
+		SizeHints.width, SizeHints.height,
+		state.total_size.x, state.total_size.y);
+#endif
         SizeHints.min_width =
         SizeHints.max_width =
-        SizeHints.width = backgroundXpm.attributes.width;
+        SizeHints.width = state.total_size.x;
         SizeHints.min_height =
         SizeHints.max_height =
-        SizeHints.height= backgroundXpm.attributes.height;
+        SizeHints.height= state.total_size.y;
 
+        backgroundWindow = XCreatePixmap(
+                mainDisplay,
+                Root,
+                state.total_size.x,
+                state.total_size.y,
+		color_depth
+                );
+                
+        drawWindow = XCreatePixmap(
+                mainDisplay,            /* display */
+                Root,                   /* parent */
+                state.total_size.x,           /* width */
+                state.total_size.y,           /* height */
+		color_depth		/* color depth */
+                );
+                
         mainWindow = XCreateSimpleWindow(
                 mainDisplay,            /* display */
                 Root,                   /* parent */
@@ -526,24 +594,16 @@ void initialize(
                 back_pix                /* background */
                 );
                 
-        XSetWMNormalHints(mainDisplay, mainWindow, &SizeHints);
+        /*XSetWMNormalHints(mainDisplay, mainWindow, &SizeHints);*/
+        XSetWMNormalHints(mainDisplay, iconWindow, &SizeHints);
         status = XClearWindow(mainDisplay, mainWindow);
+        status = XClearWindow(mainDisplay, iconWindow);
         
         status = XGetWindowAttributes(
                 mainDisplay,    /* display */
                 mainWindow,     /* window */
                 & winAttr       /* window_attributes_return */
                 );
-        status = XSetWindowBackgroundPixmap(
-                mainDisplay,            /* display */
-                mainWindow,             /* window */
-                backgroundXpm.pixmap    /* background_pixmap */
-                );
-        status = XSetWindowBackgroundPixmap(
-                mainDisplay,            /* display */
-                iconWindow,             /* window */
-                backgroundXpm.pixmap    /* background_pixmap */
-                );
                 
         status = XStringListToTextProperty(&app_name, 1, &title);
         XSetWMName(mainDisplay, mainWindow, &title);
@@ -582,6 +642,110 @@ void initialize(
                 &mainGCV
                 );
                 
+	/* Draw the background */
+	mainGCV.foreground = back_pix;
+        XChangeGC(
+                mainDisplay,
+                mainGC,
+                GCForeground,
+                &mainGCV
+                );
+	XFillRectangle(
+		mainDisplay,
+		backgroundWindow,
+		mainGC,
+		0,
+		0,
+		state.total_size.x,
+		state.total_size.y
+		);
+	/* The shadow on the sides */
+	if (pushed_in) {
+		tmp_pix = DarkenColor(state.bgcolor, 1.6, mainDisplay, Root);
+	} else {
+		tmp_pix = LightenColor(state.bgcolor, 2.0, mainDisplay, Root);
+	}
+	mainGCV.foreground = tmp_pix;
+        XChangeGC(
+                mainDisplay,
+                mainGC,
+                GCForeground,
+                &mainGCV
+                );
+	XDrawLine(
+		mainDisplay,
+		backgroundWindow,
+		mainGC,
+		0,
+		0,
+		state.total_size.x-1,
+		0
+		);
+	XDrawLine(
+		mainDisplay,
+		backgroundWindow,
+		mainGC,
+		0,
+		0,
+		0,
+		state.total_size.y-1
+		);
+	if (! pushed_in) {
+		tmp_pix = DarkenColor(state.bgcolor, 1.6, mainDisplay, Root);
+	} else {
+		tmp_pix = LightenColor(state.bgcolor, 2.0, mainDisplay, Root);
+	}
+	mainGCV.foreground = tmp_pix;
+        XChangeGC(
+                mainDisplay,
+                mainGC,
+                GCForeground,
+                &mainGCV
+                );
+	XDrawLine(
+		mainDisplay,
+		backgroundWindow,
+		mainGC,
+		state.total_size.x-1,
+		state.total_size.y,
+		state.total_size.x-1,
+		0
+		);
+	XDrawLine(
+		mainDisplay,
+		backgroundWindow,
+		mainGC,
+		state.total_size.x,
+		state.total_size.y-1,
+		0,
+		state.total_size.y-1
+		);
+	/* Draw those little points every 5 secs on the background pixmap
+	 * that usually are marked 1,2,3,4,5,6,7 etc. */
+	mainGCV.foreground = fore_pix;
+        XChangeGC(
+                mainDisplay,
+                mainGC,
+                GCForeground,
+                &mainGCV
+                );
+	for (tmp = 0; tmp < 60; tmp += 5) {
+		xrate = convert2x(tmp);
+		yrate = convert2y(tmp);
+		tmp_coord.x = rint((watch_size.x+1) * xrate) + center.x;
+		tmp_coord.y = rint((watch_size.y+1) * yrate) + center.y;
+#ifdef DEBUG
+		printf("dot at %d %d\n", tmp_coord.x, tmp_coord.y);
+#endif
+		XDrawPoint(
+			mainDisplay,
+			backgroundWindow,
+			mainGC,
+			tmp_coord.x,
+			tmp_coord.y
+			);
+	}
+
         status = XSetCommand(mainDisplay, mainWindow, argv, argc);
         
         /* Set up the event for quitting the window */
