@@ -1,11 +1,16 @@
 #ifndef _state_h_
 #define _state_h_
 
+/* coordinates structure */
+struct coordinates {
+	int x, y;
+};
+
 struct astime_state {
 	long int update_interval; /* interval (sec) to update the dial */
 	unsigned short int draw_seconds;
-	unsigned short int draw_day;
 	unsigned short int draw_date;
+	struct coordinates total_size;
 	char bgcolor[50];
 	char fgcolor[50];
 	char seccolor[50];
